unit KA.Data.KADao.DbGUIDEdit;

interface

uses
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.DB,
  Borland.Vcl.DBCtrls,
  Borland.Vcl.Controls,
  Borland.Vcl.StdCtrls,
  Borland.Vcl.Dialogs,
  KA.Data.KADao.Table;


type
  TKADaoDbGUIDEdit = class(TEdit)
  private
    { Private declarations }
    FDataLink       : TFieldDataLink;
    FAutoPost       : Boolean;
    InSetting        : Boolean;
    Procedure          DataChange(Sender: TObject);
    Procedure          UpdateData(Sender: TObject);
    Procedure          EditingChange(Sender: TObject);
    Procedure          ActiveChange(Sender: TObject);
  protected
    { Protected declarations }
    Function    GetDataField: string;
    Procedure   SetDataField(const Value: string);
    Function    GetDataSource: TDataSource;
    Procedure   SetDataSource(Value: TDataSource);

    Procedure   Change; override;
    Procedure   Loaded; override;
    Procedure   Notification(AComponent: TComponent; Operation: TOperation); override;
    Function    CheckValidGUIDChars(S:String) : Boolean;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent);override;
    Destructor  Destroy;override;
    Function    CheckValidGUID(S:String) : Boolean;
  published
    { Published declarations }
    Property    AutoPost          : Boolean     Read FAutoPost       Write FAutoPost;
    Property    DataField         : String      Read GetDataField  Write SetDataField;
    Property    DataSource        : TDataSource Read GetDataSource Write SetDataSource;
  end;

procedure Register;

implementation

Function TKADaoDbGUIDEdit.CheckValidGUIDChars(S:String) : Boolean;
Var
  L : Integer;
  X : Integer;
Begin
  Result := False;
  L      := Length(S);
  if L=0 Then Exit;
  For X := 1 To L Do
     Begin
       if NOT (S[X] In GUID_VALID_CHARS) Then Exit;
     End;
  Result := True;
End;

Function TKADaoDbGUIDEdit.CheckValidGUID(S:String) : Boolean;
Var
  L : Integer;
Begin
 Result := True;
 L      := Length(S);
 if L=0 Then Exit;
 Result := False;
 if L <> 38 Then Exit;
 if S[1] <> '{' Then Exit;
 if S[L] <> '}' Then Exit;
 if S[10] <> '-' Then Exit;
 if S[15] <> '-' Then Exit;
 if S[20] <> '-' Then Exit;
 if S[25] <> '-' Then Exit;
 Result := CheckValidGUIDChars(S);
End;

Constructor TKADaoDbGUIDEdit.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);
  FAutoPost                 := False;
  InSetting                  := False;
  FDataLink                 := TFieldDataLink.Create;
  FDataLink.Control         := Self;
  FDataLink.OnDataChange    := DataChange;
  FDataLink.OnEditingChange := EditingChange;
  FDataLink.OnUpdateData    := UpdateData;
  FDataLink.OnActiveChange  := ActiveChange;
  Width                      := Inherited Width*2;    
End;

Destructor TKADaoDbGUIDEdit.Destroy;
Begin
 FDataLink.Free;
 FDataLink := nil;
 inherited Destroy;
End;

procedure TKADaoDbGUIDEdit.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and (AComponent = DataSource) then
      Begin
        DataSource := nil;
      End;
end;

Procedure TKADaoDbGUIDEdit.Loaded;
Begin
  Inherited Loaded;
 //*****************************************************************************
End;


Procedure TKADaoDbGUIDEdit.ActiveChange(Sender: TObject);
Begin
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet.Active) Then
      Begin

      End
  Else
      Begin
        Text := '';
      End;
End;

Procedure TKADaoDbGUIDEdit.DataChange(Sender: TObject);
Var
  SGUID : String;
Begin
  ReadOnly := NOT FDataLink.CanModify;
  If  (Assigned(FDataLink.DataSource))
  And (FDataLink.Active)
  And (FDataLink.FieldName<> '')
  And (Assigned(FDataLink.Field))
  And (FDataLink.Field.DataType=ftBytes)
  And (FDataLink.Field.ValidChars = GUID_VALID_CHARS)
  And (FDataLink.DataSet is TKAdaoTable)
  And (FDataLink.Dataset.State = dsBrowse) Then
      Begin
        SGUID      := FDataLink.Field.AsString;
        InSetting  := True;
        Try
          if FDataLink.Field.IsNull Then
             Text := ''
          Else
             Text  := TKAdaoTable(FDataLink.DataSet).GetGUIDAsString(SGUID);
        Finally
          InSetting := False;
        End;
      End;
End;

Procedure TKADaoDbGUIDEdit.UpdateData(Sender: TObject);
Var
  SGUID : String;
Begin
  If  (Assigned(FDataLink.DataSource))
  And (FDataLink.Active)
  And (FDataLink.FieldName<> '')
  And (Assigned(FDataLink.Field))
  And (FDataLink.Field.DataType=ftBytes)
  And (FDataLink.Field.ValidChars = GUID_VALID_CHARS)
  And (FDataLink.DataSet is TKAdaoTable)
  And (FDataLink.CanModify) Then                                
      Begin
        if (Length(Text) = 38) Or (Length(Text) = 0) Then
           Begin
            SGUID := '';
            if (Length(Text) = 38) Then SGUID := TKAdaoTable(FDataLink.DataSet).PutGUIDInString(Text);
            FDataLink.Field.AsString := SGUID;
           End;
      End;
End;

Procedure TKADaoDbGUIDEdit.Change;
Begin
  If  NOT (CheckValidGUID(Text)) Then Exit;
  If  (Assigned(FDataLink.DataSource))
  And (FDataLink.Active)
  And (FDataLink.FieldName<> '')
  And (Assigned(FDataLink.Field))
  And (FDataLink.Field.DataType=ftBytes)
  And (FDataLink.Field.ValidChars = GUID_VALID_CHARS)
  And (FDataLink.DataSet is TKAdaoTable)
  And (NOT ReadOnly)
  And (NOT InSetting)
  And (FDataLink.CanModify)
  And (FDataLink.Edit) Then
      Begin
       FDataLink.Modified;
       FDataLink.UpdateRecord;
       if FAutoPost Then FDataLink.DataSet.Post;
      End;
End;

Procedure TKADaoDbGUIDEdit.EditingChange(Sender: TObject);
Begin
 If  (Assigned(FDataLink.DataSource))
  And (FDataLink.Active)
  And (FDataLink.FieldName<> '')
  And (Assigned(FDataLink.Field))
  And (FDataLink.Field.DataType=ftBytes)
  And (FDataLink.Field.ValidChars = GUID_VALID_CHARS)
  And (FDataLink.DataSet is TKAdaoTable)
  And (FDataLink.DataSet.State = dsInsert) Then Text := '';
End;

Function TKADaoDbGUIDEdit.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

Procedure TKADaoDbGUIDEdit.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

Function TKADaoDbGUIDEdit.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

Procedure TKADaoDbGUIDEdit.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKADaoDbGUIDEdit]);
end;

end.
