unit KA.Data.KADao.FilterByButton;

interface

uses
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.Controls,
  Borland.Vcl.Forms,
  Borland.Vcl.Dialogs,
  Borland.Vcl.StdCtrls,
  Borland.Vcl.DB,
  Borland.Vcl.DBCtrls;

type
  TKADaoFilterByButton = class(TButton)
  private
    { Private declarations }
    FDataLink     : TFieldDataLink;
    FBefore       : Boolean;
    FLastFilter   : TStringList;
  protected
    { Protected declarations }
    Function    GetDataSource: TDataSource;
    Procedure   SetDataSource(Value: TDataSource);
    Procedure   Notification(AComponent: TComponent; Operation: TOperation); override;
    Procedure   HandleClickEvent;
    Procedure   DataChange(Sender: TObject);
    Procedure   ActiveChange(Sender: TObject);
    procedure   Loaded; override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent);override;
    Destructor  Destroy;override;
    Procedure   Click; override;
  published
    { Published declarations }
    Property    DataSource  : TDataSource Read GetDataSource Write SetDataSource;
    Property    CallBefore  : Boolean     Read FBefore         Write FBefore;
  end;

procedure Register;

implementation
Uses
  KA.Data.KADao.Table,
  KA.Data.KADao.FilterByUnit;

Const dbOpenTable = 1;

Constructor TKADaoFilterByButton.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);
  Caption     := 'Filter by';
  FLastFilter:= TStringList.Create;
  FDataLink  := TFieldDataLink.Create;
  FDataLink.OnDataChange    := DataChange;
  FDataLink.OnActiveChange  := ActiveChange;
  FBefore    := True;
End;

Destructor TKADaoFilterByButton.Destroy;
Begin
  FLastFilter.Free;
  FDataLink.Free;
  FDataLink := nil;
  Inherited Destroy;
End;

Function TKADaoFilterByButton.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

Procedure TKADaoFilterByButton.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType<>dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
end;

Procedure TKADaoFilterByButton.ActiveChange(Sender: TObject);
Begin
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType<>dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

Procedure TKADaoFilterByButton.DataChange(Sender: TObject);
Begin
if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType<>dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

Procedure TKADaoFilterByButton.Loaded;
Begin
  Inherited Loaded;
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType<>dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

procedure TKADaoFilterByButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and (AComponent = DataSource) then DataSource := nil;
end;

Procedure TKADaoFilterByButton.HandleClickEvent;
Var
  NewFilter : String;
Begin
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active) Then
     Begin
      Application.CreateForm(TKADaoFilterBy, KADaoFilterBy);
      NewFilter:=TKaDaoTable(DataSource.Dataset).Filter;
      if KADaoFilterBy.Execute(TKaDaoTable(DataSource.Dataset), NewFilter, FLastFilter) Then
         Begin
           if NewFilter  <> '' Then
              Begin
               TKaDaoTable(DataSource.Dataset).Filter   := NewFilter;
               TKaDaoTable(DataSource.Dataset).Filtered := True;
              End
           Else
              Begin
               TKaDaoTable(DataSource.Dataset).Filtered := False;
              End;
         End;
      KADaoFilterBy.Free;
     End;
End;

Procedure TKADaoFilterByButton.Click;
Begin
 if FBefore Then inherited Click;
 HandleClickEvent;
 if NOT FBefore Then inherited Click;
End;

procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKADaoFilterByButton]);
end;

end.
