unit KA.Data.KADao.HRollForm;
interface

uses
  Borland.Vcl.Windows,
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.Controls,
  Borland.Vcl.Forms,
  Borland.Vcl.ExtCtrls;

Const
  BevelCount = 5;
type
  TRollType    = (rtLeft, rtRight, rtTop, rtBottom);
  TKABevel     = class(TBevel)
  protected
    procedure   DblClick; override;
  End;

  TKAHRollForm = class(TCustomPanel)
  private
    { Private declarations }
    FBevels        : Array[1..BevelCount] of TBevel;
    FExpandedWidth : Integer;
    FShrinkedWidth : Integer;
    FBevelWidth    : Integer;
    FControls      : TList;
    FRollType      : TRollType;
    FRollAtStartup : Boolean;
    FRollAnimation : Boolean;
    FRollStep      : Integer;
    FRollControls  : Boolean;
    Procedure         SetRollType(Value : TRollType);
  protected
    { Protected declarations }
    Function    IsMyBevel(Ctrl:TControl):Boolean;
    Procedure   HideAll;
    Procedure   ShowAll;
    Procedure   MoveAllToTheLeft;
    Procedure   MoveAllToTheRight;
    Procedure   MoveAllToTheTop;
    Procedure   MoveAllToTheBottom;

    Procedure   MoveAllToTheLeftByStep(Step_:Integer);
    Procedure   MoveAllToTheRightByStep(Step_:Integer);
    Procedure   MoveAllToTheTopByStep(Step_:Integer);
    Procedure   MoveAllToTheBottomByStep(Step_:Integer);

    Procedure   HorizontalAnimeRoll(From_, To_, Step : Integer);
    Procedure   VerticalAnimeRoll(From_, To_, Step : Integer);
    Procedure   Loaded; override;
    Procedure   DoublClick;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent);override;
    Destructor  Destroy;override;

    property    Align;
    property    Alignment;
    property    Anchors;
    property    AutoSize;
    property    BevelInner;
    property    BevelOuter;
    property    BevelWidth;
    property    BiDiMode;
    property    BorderWidth;
    property    BorderStyle;
    property    Caption;
  published
    { Published declarations }
    Property RollType      : TRollType Read FRollType      Write SetRollType;
    Property RollAtStartup : Boolean   Read FRollAtStartup Write FRollAtStartup;
    Property RollAnimation : Boolean   Read FRollAnimation Write FRollAnimation;
    Property RollStep      : Integer   Read FRollStep      Write FRollStep;
    Property RollControls  : Boolean   Read FRollControls  Write FRollControls;
    property Color;
    property Constraints;
    property Ctl3D;
    property UseDockManager;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FullRepaint;
    property Font;
    property Locked;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDockDrop;
    property OnDockOver;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
  end;

procedure Register;

implementation
uses dialogs;

procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKAHRollForm]);
end;

{ TKAHRollForm }

procedure TKABevel.DblClick;
Begin
  if Owner is TKAHRollForm Then TKAHRollForm(Owner).DoublClick;
End;

constructor TKAHRollForm.Create(AOwner: TComponent);
Var
  X : Integer;
begin
  inherited Create(AOwner);
  ControlStyle        := ControlStyle - [csSetCaption];
  ControlStyle        := ControlStyle - [csFramed];
  FRollType          := rtRight;
  FRollAtStartup     := True;
  FRollAnimation     := False;
  FRollControls      := True;
  FRollStep          := 2;
  Align               := alRight;
  FControls          := TList.Create;
  FShrinkedWidth     := GetSystemMetrics(SM_CXSIZE);
  FBevelWidth        := FShrinkedWidth Div BevelCount;
  For X := 1 To BevelCount do
      Begin
        FBevels[X]        := TKABevel.Create(Self);
        FBevels[X].Style  := bsRaised;
        FBevels[X].Align  := AlRight;
        FBevels[X].Width  := FBevelWidth;
        FBevels[X].Parent := Self;
      End;
  Self.BevelInner  := bvNone;
  Self.BevelOuter  := bvNone;
  Self.FullRepaint := False;
end;

destructor TKAHRollForm.Destroy;
begin
  FControls.Free;
  inherited Destroy;
end;

Function TKAHRollForm.IsMyBevel(Ctrl:TControl):Boolean;
Var
 X : Integer;
Begin
 Result := True;
 For X  := 1 to BevelCount do If FBevels[X]=Ctrl Then Exit;
 Result := False;
End;

procedure TKAHRollForm.HideAll;
Var
  X : Integer;
begin
  FControls.Clear;
  For X := 0 To Self.ControlCount-1 do
      Begin
       if Self.Controls[X].Visible Then
          Begin
            if Not IsMyBevel(Self.Controls[X]) Then
               Begin
                 FControls.Add(Self.Controls[X]);
                 Self.Controls[X].Visible:= False;
               End;
          End;
      End;
end;

procedure TKAHRollForm.ShowAll;
Var
  X : Integer;
begin
  For X := 0 To FControls.Count-1 do
      Begin
        TControl(FControls.Items[X]).Visible := True;
      End;
end;

procedure TKAHRollForm.MoveAllToTheLeft;
Var
  X : Integer;
Begin
 if Owner is TForm Then
     Begin
       For X := 0 To TForm(Owner).ControlCount-1 do
           Begin
            if TForm(Owner).Controls[X] <> Self Then
               TForm(Owner).Controls[X].Left := TForm(Owner).Controls[X].Left-(FExpandedWidth-FShrinkedWidth);
           End;
     End;
End;

procedure TKAHRollForm.MoveAllToTheRight;
Var
  X : Integer;
Begin
 if Owner is TForm Then
     Begin
       For X := 0 To TForm(Owner).ControlCount-1 do
           Begin
            if TForm(Owner).Controls[X] <> Self Then
               TForm(Owner).Controls[X].Left := TForm(Owner).Controls[X].Left+(FExpandedWidth-FShrinkedWidth);
           End;
     End;
End;

procedure TKAHRollForm.MoveAllToTheTop;
Var
  X : Integer;
Begin
 if Owner is TForm Then
     Begin
       For X := 0 To TForm(Owner).ControlCount-1 do
           Begin
            if TForm(Owner).Controls[X] <> Self Then
               TForm(Owner).Controls[X].Top := TForm(Owner).Controls[X].Top-(FExpandedWidth-FShrinkedWidth);
           End;
     End;
End;

procedure TKAHRollForm.MoveAllToTheBottom;
Var
  X : Integer;
Begin
 if Owner is TForm Then
     Begin
       For X := 0 To TForm(Owner).ControlCount-1 do
           Begin
            if TForm(Owner).Controls[X] <> Self Then
               Begin
                 TForm(Owner).Controls[X].Top := TForm(Owner).Controls[X].Top+(FExpandedWidth-FShrinkedWidth);
               End;
           End;
     End;
End;


procedure TKAHRollForm.MoveAllToTheLeftByStep(Step_:Integer);
Var
  X : Integer;
Begin
 if Owner is TForm Then
     Begin
       For X := 0 To TForm(Owner).ControlCount-1 do
           Begin
            if TForm(Owner).Controls[X] <> Self Then
               Begin
                 TForm(Owner).Controls[X].Left := TForm(Owner).Controls[X].Left-Step_;
               End;
           End;
     End;
End;

procedure TKAHRollForm.MoveAllToTheRightByStep(Step_:Integer);
Var
  X : Integer;
Begin
 if Owner is TForm Then
     Begin
       For X := 0 To TForm(Owner).ControlCount-1 do
           Begin
            if TForm(Owner).Controls[X] <> Self Then
               Begin
                 TForm(Owner).Controls[X].Left := TForm(Owner).Controls[X].Left+Step_;
               End;
           End;
     End;
End;

procedure TKAHRollForm.MoveAllToTheTopByStep(Step_:Integer);
Var
  X : Integer;
Begin
 if Owner is TForm Then
     Begin
       For X := 0 To TForm(Owner).ControlCount-1 do
           Begin
            if TForm(Owner).Controls[X] <> Self Then
               Begin
                 TForm(Owner).Controls[X].Top := TForm(Owner).Controls[X].Top-Step_;
               End;
           End;
     End;
End;

procedure TKAHRollForm.MoveAllToTheBottomByStep(Step_:Integer);
Var
  X : Integer;
Begin
 if Owner is TForm Then
     Begin
       For X := 0 To TForm(Owner).ControlCount-1 do
           Begin
            if TForm(Owner).Controls[X] <> Self Then
               Begin
                 TForm(Owner).Controls[X].Top := TForm(Owner).Controls[X].Top+Step_;
               End;
           End;
     End;
End;

Procedure TKAHRollForm.Loaded;
Begin
  Inherited Loaded;
  if csDesigning in ComponentState Then Exit;
  if Owner is TForm Then
     Begin
       if RollAtStartup Then HideAll;
       if FRollType=rtLeft Then
          Begin
            FExpandedWidth    := Width;
            if RollAtStartup Then
               Begin
                 TForm(Owner).Width := TForm(Owner).Width-(Self.Width-FShrinkedWidth);
                 Width              := FShrinkedWidth;
                 MoveAllToTheLeft;
               End;
          End
       Else
       if FRollType=rtRight Then
          Begin
            FExpandedWidth    := Width;
            if RollAtStartup Then
               Begin
                 TForm(Owner).Width := TForm(Owner).Width-(Self.Width-FShrinkedWidth);
                 Width              := FShrinkedWidth;
               End;
          End
       Else
       if FRollType=rtTop Then
          Begin
            FExpandedWidth     := Height;
            if RollAtStartup Then
               Begin
                 TForm(Owner).Height := TForm(Owner).Height-(Self.Height-FShrinkedWidth);
                 Height              := FShrinkedWidth;
                 MoveAllToTheTop;
               End;
          End
       Else
       if FRollType=rtBottom Then
          Begin
            FExpandedWidth     := Height;
            if RollAtStartup Then
               Begin
                 TForm(Owner).Height := TForm(Owner).Height-(Self.Height-FShrinkedWidth);
                 Height              := FShrinkedWidth;
               End;
          End;
     End;
End;

procedure TKAHRollForm.SetRollType(Value: TRollType);
Var
  X : Integer;
begin
 if FRollType <> Value Then
    Begin
      FRollType := Value;
      if FRollType=rtLeft Then
         Begin
           Align := alLeft;
           For X := 1 To BevelCount do
               Begin
                 FBevels[X].Align  := AlLeft;
               End;
         End
      Else
      if FRollType=rtRight Then
         Begin
           Align := alRight;
           For X := 1 To BevelCount do
               Begin
                 FBevels[X].Align  := AlRight;
               End;
         End
      Else
      if FRollType=rtTop Then
         Begin
           Align := alTop;
           For X := 1 To BevelCount do
               Begin
                 FBevels[X].Align  := AlTop;
               End;
         End
      Else
      if FRollType=rtBottom Then
         Begin
           Align := alBottom;
           For X := 1 To BevelCount do
               Begin
                 FBevels[X].Align  := AlBottom;
               End;
         End;
    End;
end;

Procedure TKAHRollForm.HorizontalAnimeRoll(From_, To_, Step : Integer);
Var
 X : Integer;
Begin
  X     := (To_   Div Step)*Step;
  From_ := (From_ Div Step)*Step;
  Repeat
    TForm(Owner).Width := TForm(Owner).Width+Step;
    if FRollControls Then
       Begin
         Width := Width+Step;
         If FRollType=rtLeft Then
            Begin
              if Step > 0 Then
                 MoveAllToTheRightByStep(Step)
               Else
                 MoveAllToTheLeftByStep(-Step);
            End;
       End;
    From_ := From_+Step;
    Application.ProcessMessages;
  Until From_=X;
End;


Procedure TKAHRollForm.VerticalAnimeRoll(From_, To_, Step : Integer);
Var
 X  : Integer;
Begin
  X     := (To_   Div Step)*Step;
  From_ := (From_ Div Step)*Step;
  Repeat
    TForm(Owner).Height := TForm(Owner).Height+Step;
    if FRollControls Then
       Begin
         Height := Height+Step;
         If FRollType=rtTop Then
            Begin
              if Step > 0 Then
                 MoveAllToTheBottomByStep(Step)
               Else
                 MoveAllToTheTopByStep(-Step);
            End;
       End;
    From_ := From_+Step;
    Application.ProcessMessages;
  Until From_=X;
End;


procedure TKAHRollForm.DoublClick;
Var
 Temp : Integer;
begin
  if Not FRollAnimation Then FRollControls := False;
  if Owner is TForm Then
     Begin
      if FRollType=rtLeft Then
         Begin
           if Width=FShrinkedWidth Then
              Begin
                Temp               := TForm(Owner).Width;
                if FRollControls  Then ShowAll;
                If FRollAnimation Then
                   HorizontalAnimeRoll(TForm(Owner).Width,TForm(Owner).Width+(FExpandedWidth-FShrinkedWidth),FRollStep);
                TForm(Owner).Width := Temp+(FExpandedWidth-FShrinkedWidth);
                Width              := FExpandedWidth;
                if NOT FRollControls  Then
                   Begin
                     MoveAllToTheRight;
                     ShowAll;
                   End;
              End
           Else
              Begin
                Temp               := TForm(Owner).Width;
                if NOT FRollControls Then
                   Begin
                     HideAll;
                     Width              := FShrinkedWidth;
                   End;
                If FRollAnimation Then
                   HorizontalAnimeRoll(TForm(Owner).Width,TForm(Owner).Width-(FExpandedWidth-FShrinkedWidth),-FRollStep);
                TForm(Owner).Width := Temp-(FExpandedWidth-FShrinkedWidth);
                if NOT FRollControls Then MoveAllToTheLeft;
                if FRollControls Then
                   Begin
                     HideAll;
                     Width              := FShrinkedWidth;
                   End;
              End;
         End
      Else
      if FRollType=rtRight Then
         Begin
           if Width=FShrinkedWidth Then
              Begin
                Temp               := TForm(Owner).Width;
                if FRollControls  Then ShowAll;
                If FRollAnimation Then
                   HorizontalAnimeRoll(TForm(Owner).Width,TForm(Owner).Width+(FExpandedWidth-FShrinkedWidth),FRollStep);
                TForm(Owner).Width := Temp+(FExpandedWidth-FShrinkedWidth);
                Width              := FExpandedWidth;
                if NOT FRollControls  Then ShowAll;
              End
           Else
              Begin
                Temp               := TForm(Owner).Width;
                if NOT FRollControls Then
                   Begin
                     HideAll;
                     Width              := FShrinkedWidth;
                   End;
                If FRollAnimation Then
                   HorizontalAnimeRoll(TForm(Owner).Width,TForm(Owner).Width-(FExpandedWidth-FShrinkedWidth),-FRollStep);
                TForm(Owner).Width := Temp-(FExpandedWidth-FShrinkedWidth);
                if FRollControls Then
                   Begin
                     HideAll;
                     Width              := FShrinkedWidth;
                   End;
              End;
         End
      Else
      if FRollType=rtTop Then
         Begin
           if Height=FShrinkedWidth Then
              Begin
                Temp                := TForm(Owner).Height;
                if FRollControls  Then ShowAll;
                If FRollAnimation Then
                   VerticalAnimeRoll( TForm(Owner).Height, TForm(Owner).Height+(FExpandedWidth-FShrinkedWidth),FRollStep);
                TForm(Owner).Height := Temp+(FExpandedWidth-FShrinkedWidth);
                if NOT FRollControls Then MoveAllToTheBottom;
                Height              := FExpandedWidth;
                if NOT FRollControls Then ShowAll;
              End
           Else
              Begin
                Temp                := TForm(Owner).Height;
                if NOT FRollControls Then
                   Begin
                     HideAll;
                     Height              := FShrinkedWidth;
                   End;
                If FRollAnimation Then
                   VerticalAnimeRoll( TForm(Owner).Height, TForm(Owner).Height-(FExpandedWidth-FShrinkedWidth),-FRollStep);
                TForm(Owner).Height := Temp-(FExpandedWidth-FShrinkedWidth);
                if NOT FRollControls Then MoveAllToTheTop;
                if FRollControls Then
                   Begin
                     HideAll;
                     Height              := FShrinkedWidth;
                   End;
              End;
         End
      Else
      if FRollType=rtBottom Then
         Begin
           if Height=FShrinkedWidth Then
              Begin
                Temp                := TForm(Owner).Height;
                if FRollControls Then ShowAll;
                If FRollAnimation Then
                   VerticalAnimeRoll( TForm(Owner).Height, TForm(Owner).Height+(FExpandedWidth-FShrinkedWidth),FRollStep);
                TForm(Owner).Height := Temp+(FExpandedWidth-FShrinkedWidth);
                Height              := FExpandedWidth;
                if NOT FRollControls Then ShowAll;
              End
           Else
              Begin
                Temp                := TForm(Owner).Height;
                if NOT FRollControls Then
                   Begin
                     HideAll;
                     Height              := FShrinkedWidth;
                   End;
                If FRollAnimation Then
                   VerticalAnimeRoll( TForm(Owner).Height, TForm(Owner).Height-(FExpandedWidth-FShrinkedWidth),-FRollStep);
                TForm(Owner).Height := Temp-(FExpandedWidth-FShrinkedWidth);
                if FRollControls Then
                   Begin
                     HideAll;
                     Height              := FShrinkedWidth;
                   End;
              End;
         End;
     End;
end;



end.
