unit KA.Data.KADao.IncrementalSearch;

interface

uses
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.Controls,
  Borland.Vcl.StdCtrls,
  Borland.Vcl.ExtCtrls,
  Borland.Vcl.DB,
  Borland.Vcl.Variants,
  Borland.Vcl.DBCtrls,
  Borland.Vcl.Dialogs,
  KA.Data.KADao.Table;


type
  TKADaoIncrementalSearch = class(TEdit)
  private
    { Private declarations }
    FDataLink          : TFieldDataLink;
    FAutoComplete      : Boolean;
    FEnteredText       : String;
    Seek                : Boolean;
    Skip                : Boolean;
  protected
    { Protected declarations }
    Function    GetDataField: string;
    Procedure   SetDataField(const Value: string);
    Function    GetDataSource: TDataSource;
    Procedure   SetDataSource(Value: TDataSource);
    Procedure   Notification(AComponent: TComponent; Operation: TOperation); override;
    Procedure   Check;
    Procedure   DataChange(Sender: TObject);
    Procedure   ActiveChange(Sender: TObject);
    procedure   Loaded; override;
    Procedure   Change; override;
    Procedure   KeyPress(var Key: Char);override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent);override;
    Destructor  Destroy;override;
  published
    { Published declarations }
    Property    DataField          : String      Read GetDataField      Write SetDataField;
    Property    DataSource         : TDataSource Read GetDataSource     Write SetDataSource;
    Property    AutoComplete       : Boolean     Read FAutoComplete       Write FAutoComplete;
  end;

procedure Register;

implementation
Const
  LeftStart   = 10;
  TopStart    = 10;
  Spacing     = 5;
  dbOpenTable = 1;

Constructor TKADaoIncrementalSearch.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);
  FDataLink                 := TFieldDataLink.Create;
  FDataLink.OnDataChange    := DataChange;
  FDataLink.OnActiveChange  := ActiveChange;
  FAutoComplete             := False;
  FEnteredText              := '';
  Skip                       := False;
End;

Destructor TKADaoIncrementalSearch.Destroy;
Begin
  FDataLink.Free;
  Inherited Destroy;
End;

Procedure TKADaoIncrementalSearch.Check;
Var
  DaoTable : TKADaoTable;
Begin
  DaoTable := TKADaoTable(FDataLink.Dataset);
  Seek := False;
  if (DaoTable.TableType=dbOpenTable) Then
     Begin
       if DaoTable.IndexDefs.Count=0 Then
          Begin
            Enabled := False;
            Exit;
          End;
       Seek := True;
     End;
End;

procedure TKADaoIncrementalSearch.Change;
Var
 KeyFields : String;
 KeyValues : Variant;
 Options   : TLocateOptions;
 DaoTable  : TKADaoTable;
 OK        : Boolean;
 OAFI      : Boolean;
 CP        : Integer;
begin
 if Skip Then
    Begin
     Skip := False;
     CP := GetSelStart+GetSelLength;
     FEnteredText := Copy(Text,1,CP);
     Exit;
    End;
 KeyFields     := FDataLink.FieldName;
 if FAutoComplete Then
    Begin
      CP := GetSelStart+GetSelLength;
      FEnteredText := Copy(Text,1,CP);
    End
 Else
    Begin
      FEnteredText := Text;
    End;
 KeyValues     := FEnteredText;
 Options       := [loCaseInsensitive];
 DaoTable      := TKADaoTable(FDataLink.Dataset);
 OK            := False;
 if NOT Seek Then
    Begin
      Try
        OK := DaoTable.Find_First(KeyFields, KeyValues, Options);
        if Not OK Then
           Begin
             Options := Options+[loPartialKey];
             OK := DaoTable.Find_First(KeyFields, KeyValues, Options);
           End;
      Except
      End;
      VarClear(KeyValues);
    End
 Else
    Begin
      OAFI := DaoTable.AutoFindIndex;
      DaoTable.AutoFindIndex := True;
      Try
        OK := DaoTable.Locate(KeyFields, KeyValues, Options);
        if Not OK Then
           Begin
             Options := Options+[loPartialKey];
             OK := DaoTable.Locate(KeyFields, KeyValues, Options);
           End;
      Except
      End;
      DaoTable.AutoFindIndex := OAFI;
      VarClear(KeyValues);
    End;
 if (OK) And (FAutoComplete) Then
    Begin
      Skip := True;
      CP := GetSelStart+GetSelLength;
      Self.Text := FDataLink.DataSet.FieldByName(KeyFields).AsString;
      Self.SetSelStart(CP);
      Skip := False;
    End;
End;

Procedure TKADaoIncrementalSearch.KeyPress(var Key: Char);
Var
 KeyFields : string;
 KeyValues : Variant;
 Options   : TLocateOptions;
 DaoTable  : TKADaoTable;
 OK        : Boolean;
 CP        : Integer;
Begin
 Inherited KeyPress(Key);
 if Key <> #13 Then Exit;
 KeyValues     := FEnteredText;
 KeyFields     := FDataLink.FieldName;
 if FAutoComplete Then
    Begin
      CP := GetSelStart+GetSelLength;
      FEnteredText := Copy(Text,1,CP);
    End
 Else
    Begin
      FEnteredText := Text;
    End;
 Options       := [loCaseInsensitive];
 DaoTable      := TKADaoTable(FDataLink.Dataset);
 if NOT Seek Then
    Begin
      Try
        OK := DaoTable.Find_Next(KeyFields, KeyValues, Options);
        if Not OK Then
           Begin
             Options := Options+[loPartialKey];
             DaoTable.Find_Next(KeyFields, KeyValues, Options);
           End;
      Except
      End;
      VarClear(KeyValues);
    End;
End;

Function TKADaoIncrementalSearch.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

Procedure TKADaoIncrementalSearch.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

Function TKADaoIncrementalSearch.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

Procedure TKADaoIncrementalSearch.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active) Then
      Begin
        Enabled := True;
        Check;
      End
  Else
      Begin
        Enabled := False;
      End;
end;

Procedure TKADaoIncrementalSearch.ActiveChange(Sender: TObject);
Begin
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active) Then
      Begin
        Enabled := True;
        Check;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

Procedure TKADaoIncrementalSearch.DataChange(Sender: TObject);
Begin
if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active) Then
      Begin
        Enabled := True;
        Check;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

Procedure TKADaoIncrementalSearch.Loaded;
Begin
  Inherited Loaded;
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active) Then
      Begin
        Enabled := True;
        Check;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

procedure TKADaoIncrementalSearch.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and (AComponent = DataSource) then DataSource := nil;
end;

procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKADaoIncrementalSearch]);
end;

end.
