unit KA.Data.KADao.IndexesCombo;
interface

uses
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.DB,
  Borland.Vcl.DBCtrls,
  Borland.Vcl.StdCtrls,
  KA.Data.KADao.Table,
  Borland.Vcl.Dialogs;

type
  TKADaoIndexesCombo = class(TComboBox)
  private
    { Private declarations }
    FDataLink       : TFieldDataLink;
    FAutoOpen       : Boolean;
    FInGetting      : Boolean;
    Procedure          ActiveChange(Sender: TObject);
  protected
    { Protected declarations }
    Function    GetDataSource: TDataSource;
    Procedure   SetDataSource(Value: TDataSource);
    Procedure   SetAutoOpen(Value: Boolean);
    Procedure   Notification(AComponent: TComponent; Operation: TOperation); override;
    Procedure   Select;override;
    Procedure   Loaded; override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent);override;
    Destructor  Destroy;override;
  published
    { Published declarations }
    Property    AutoOpen          : Boolean     Read FAutoOpen        Write SetAutoOpen;
    Property    DataSource        : TDataSource Read GetDataSource  Write SetDataSource;
  end;

procedure Register;

implementation
Const
  dbOpenTable = 1;

Constructor TKADaoIndexesCombo.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);
  FAutoOpen                 := True;
  FInGetting                := False;
  FDataLink                 := TFieldDataLink.Create;
  FDataLink.Control         := Self;
  FDataLink.OnActiveChange  := ActiveChange;
  Style                      := csDropDownList;
End;

Destructor TKADaoIndexesCombo.Destroy;
Begin
 FDataLink.Free;
 FDataLink := nil;
 inherited Destroy;
End;

procedure TKADaoIndexesCombo.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and (AComponent = DataSource) then
      Begin
        DataSource := nil;
      End;
end;

Procedure TKADaoIndexesCombo.Loaded;
Begin
 Inherited Loaded;
 Items.Clear;
 Text := '';
End;

Procedure TKADaoIndexesCombo.Select;
Var
  II : Integer;
Begin
  If  (Assigned(FDataLink.DataSource))
  And (DataSource.Dataset is TKaDaoTable)
  And (TKaDaoTable(DataSource.Dataset).TableType=dbOpenTable)
  And (FAutoOpen)
  And (FDataLink.Active)
  And (NOT FInGetting) Then
      Begin
        II := Items.IndexOf(Text);
        if II > -1 Then
           Begin
             FInGetting := True;
             TKaDaoTable(DataSource.Dataset).IndexName := Text;
             ItemIndex := II;
             FInGetting := False;
           End;
      End;
    Inherited Select;    
End;

Procedure TKADaoIndexesCombo.ActiveChange(Sender: TObject);
Var
  II  : Integer;
  Txt : String;
  T   : TStringList;
Begin
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.Dataset.Active)
  And (DataSource.Dataset is TKaDaoTable)
  And (TKaDaoTable(DataSource.Dataset).TableType=dbOpenTable)
  And (NOT FInGetting) Then
      Begin
        Text := '';
        T    := TStringList.Create;
        Try
         TKaDaoTable(DataSource.Dataset).GetIndexNames(T);
         Txt := T.Text;
        Finally
         T.Free;
        End;
        if Txt <> Items.Text Then Items.Text := Txt;
        II := Items.IndexOf(TKaDaoTable(DataSource.Dataset).IndexName);
        if II <> -1 Then ItemIndex := II;
      End;
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.Dataset is TKaDaoTable)
  And (NOT DataSource.DataSet.Active)
  And (NOT FInGetting) Then
      Begin
        Items.Clear;
      End;
End;

Function TKADaoIndexesCombo.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

Procedure TKADaoIndexesCombo.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

Procedure TKADaoIndexesCombo.SetAutoOpen(Value: Boolean);
Begin
 FAutoOpen := Value;
 Items.Clear;
 ActiveChange(Self);
End;



procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKADaoIndexesCombo]);
end;

end.
