unit KA.Data.KADao.Search;

interface

uses
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.Controls,
  Borland.Vcl.StdCtrls,
  Borland.Vcl.ExtCtrls,
  Borland.Vcl.DB,
  Borland.Vcl.DBCtrls,
  Borland.Vcl.Variants,
  KA.Data.KADao.Table;

type
  TKADaoSearch = class(TPanel)
  private
    { Private declarations }
    FSLabel            : TLabel;
    FSCombo            : TComboBox;
    FSEdit             : TEdit;
    FSButton           : TButton;
    FDataLink          : TFieldDataLink;
    Seek                : Boolean;
    FIncrementalSearch : Boolean;
  protected
    { Protected declarations }
    Procedure   FillCombo;
    Function    FGet_DataSource: TDataSource;
    Procedure   FSet_DataSource(Value: TDataSource);
    Procedure   Notification(AComponent: TComponent; Operation: TOperation); override;
    Procedure   DataChange(Sender: TObject);
    Procedure   ActiveChange(Sender: TObject);
    procedure   Loaded; override;
    Procedure   OnSearchClick(Sender: TObject);
    Procedure   OnEditChange(Sender: TObject);
    Property    SearchEdit   : TEdit     Read FSEdit   Write FSEdit;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent);override;
    Destructor  Destroy;override;
    Property    SearchLabel  : TLabel    Read FSLabel  Write FSLabel;
    Property    SearchCombo  : TComboBox Read FSCombo  Write FSCombo;
    Property    SearchButton : TButton   Read FSButton Write FSButton;
  published
    { Published declarations }
    Property    IncrementalSearch  : Boolean     Read FIncrementalSearch  Write FIncrementalSearch;
    Property    DataSource         : TDataSource Read FGet_DataSource     Write FSet_DataSource;
  end;

procedure Register;

implementation
Const
  LeftStart   = 10;
  TopStart    = 10;
  Spacing     = 5;
  dbOpenTable = 1;

Constructor TKADaoSearch.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);

  ControlStyle := ControlStyle - [csSetCaption];
  BevelInner   := bvLowered;
  BevelInner   := bvLowered;
  BevelOuter   := bvRaised;

  FSLabel     := TLabel.Create(Self);
  FSCombo     := TComboBox.Create(Self);
  FSEdit      := TEdit.Create(Self);
  FSButton    := TButton.Create(Self);

  FSLabel.Parent  := Self;
  FSCombo.Parent  := Self;
  FSEdit.Parent   := Self;
  FSButton.Parent := Self;

  FSLabel.Left    := LeftStart;
  FSLabel.Top     := TopStart+((FSCombo.Height-FSLabel.Height) div 2);
  FSLabel.Caption := 'Field :';

  FSCombo.Left    := FSLabel.Left+FSLabel.Width+Spacing;
  FSCombo.Top     := TopStart;
  FSCombo.Style   := csDropDownList;

  FSEdit.Left       := FSCombo.Left+FSCombo.Width+Spacing;
  FSEdit.Top        := TopStart;
  FSEdit.OnChange   := OnEditChange;

  FSButton.Left      := FSEdit.Left+FSEdit.Width+Spacing;
  FSButton.Top       := TopStart-((FSButton.Height-FSEdit.Height) div 2);
  FSButton.Caption   := 'Search';
  FSButton.Default   := True;
  FSButton.OnClick   := OnSearchClick;

  Width                     := FSButton.Left+FSButton.Width+Spacing+BevelWidth*6;
  Height                    := FSButton.Height+(TopStart)+(BevelWidth*6);

  FDataLink                 := TFieldDataLink.Create;
  FDataLink.OnDataChange    := DataChange;
  FDataLink.OnActiveChange  := ActiveChange;

  FIncrementalSearch        := False;
End;

Destructor TKADaoSearch.Destroy;
Begin
  FSLabel.Free;
  FSCombo.Free;
  FSEdit.Free;
  FSButton.Free;
  FDataLink.Free;
  Inherited Destroy;
End;

Procedure TKADaoSearch.FillCombo;
Var
  DaoTable : TKADaoTable;
  X        : Integer;
Begin
  DaoTable := TKADaoTable(FDataLink.Dataset);
  Seek := False;
  if (DaoTable.TableType=dbOpenTable) Then
     Begin
       if DaoTable.IndexDefs.Count=0 Then
          Begin
            Enabled := False;
            Exit;
          End;
       Seek := True;
     End;
  FSCombo.Items.Clear;
  if NOT Seek Then
    Begin
     For X := 0 to DaoTable.FieldCount-1 do
      Begin
       if (DaoTable.Fields[X].FieldKind=fkData) And (DaoTable.Fields[X].Visible)  Then
         Begin
          FSCombo.Items.Add(DaoTable.Fields[X].FieldName);
         End;
      End;
      FSCombo.ItemIndex := 0;
    End
 Else
    Begin
     For X := 0 to DaoTable.IndexFieldCount-1 do
      Begin
       if (DaoTable.IndexFields[X].FieldKind=fkData)  And (DaoTable.IndexFields[X].Visible) Then
         Begin
          FSCombo.Items.Add(DaoTable.IndexFields[X].FieldName);
         End;
      End;
      FSCombo.ItemIndex := 0;
      if FSCombo.ItemIndex = -1 Then FSCombo.ItemIndex:=0;
    End;
End;

procedure TKADaoSearch.OnSearchClick(Sender: TObject);
Var
 KeyFields : string;
 KeyValues : Variant;
 Options   : TLocateOptions;
 DaoTable  : TKADaoTable;
 OK        : Boolean;
 OAFI      : Boolean;
begin
 KeyFields := FSCombo.Items.Strings[FSCombo.ItemIndex];
 KeyValues := FSEdit.Text;
 Options   := [loCaseInsensitive];
 DaoTable  := TKADaoTable(FDataLink.Dataset);
 if NOT Seek Then
    Begin
      Try
        OK := DaoTable.Find_First(KeyFields, KeyValues, Options);
        if Not OK Then
           Begin
             Options := Options+[loPartialKey];
             DaoTable.Find_First(KeyFields, KeyValues, Options);
           End;
      Except
      End;
      VarClear(KeyValues);
    End
 Else
    Begin
      OAFI := DaoTable.AutoFindIndex;
      DaoTable.AutoFindIndex := True;
      Try
        OK := DaoTable.Locate(KeyFields, KeyValues, Options);
        if Not OK Then
           Begin
             Options := Options+[loPartialKey];
             DaoTable.Locate(KeyFields, KeyValues, Options);
           End;
      Except
      End;
      DaoTable.AutoFindIndex := OAFI;
      VarClear(KeyValues);
    End;
end;

procedure TKADaoSearch.OnEditChange(Sender: TObject);
Begin
 if FIncrementalSearch Then FSButton.Click;
End;

Function TKADaoSearch.FGet_DataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

Procedure TKADaoSearch.FSet_DataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active) Then
      Begin
        Enabled := True;
        FillCombo;
      End
  Else
      Begin
        Enabled := False;
      End;
end;

Procedure TKADaoSearch.ActiveChange(Sender: TObject);
Begin
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active) Then
      Begin
        Enabled := True;
        FillCombo;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

Procedure TKADaoSearch.DataChange(Sender: TObject);
Begin
if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

Procedure TKADaoSearch.Loaded;
Begin
  Inherited Loaded;
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active) Then
      Begin
        Enabled := True;
        FillCombo;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

procedure TKADaoSearch.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and (AComponent = DataSource) then DataSource := nil;
end;

procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKADaoSearch]);
end;

end.
