unit KA.Data.KADao.SortByButton;

interface

uses
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.Controls,
  Borland.Vcl.Forms,
  Borland.Vcl.Dialogs,
  Borland.Vcl.StdCtrls,
  Borland.Vcl.DB,
  Borland.Vcl.DBCtrls;

type
  TKADaoSortByButton = class(TButton)
  private
    { Private declarations }
    FDataLink     : TFieldDataLink;
    FBefore       : Boolean;
  protected
    { Protected declarations }
    Function    GetDataSource: TDataSource;
    Procedure   SetDataSource(Value: TDataSource);
    Procedure   Notification(AComponent: TComponent; Operation: TOperation); override;
    Procedure   HandleClickEvent;
    Procedure   DataChange(Sender: TObject);
    Procedure   ActiveChange(Sender: TObject);
    procedure   Loaded; override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent);override;
    Destructor  Destroy;override;
    Procedure   Click; override;
  published
    { Published declarations }
    Property    DataSource  : TDataSource Read GetDataSource Write SetDataSource;
    Property    CallBefore  : Boolean     Read FBefore         Write FBefore;
  end;

procedure Register;

implementation
Uses
  KA.Data.KADao.Table,
  KA.Data.KADao.SortByDialogUnit;

Const
  dbOpenTable = 1;
  
Constructor TKADaoSortByButton.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);
  Caption     := 'Sort by';
  FDataLink  := TFieldDataLink.Create;
  FDataLink.OnDataChange    := DataChange;
  FDataLink.OnActiveChange  := ActiveChange;
  FBefore    := True;
End;

Destructor TKADaoSortByButton.Destroy;
Begin
  FDataLink.Free;
  FDataLink := nil;
  Inherited Destroy;
End;

Function TKADaoSortByButton.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

Procedure TKADaoSortByButton.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType<>dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
end;

Procedure TKADaoSortByButton.ActiveChange(Sender: TObject);
Begin
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType<>dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

Procedure TKADaoSortByButton.DataChange(Sender: TObject);
Begin
if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType<>dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

Procedure TKADaoSortByButton.Loaded;
Begin
  Inherited Loaded;
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType<>dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

procedure TKADaoSortByButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and (AComponent = DataSource) then DataSource := nil;
end;

Procedure TKADaoSortByButton.HandleClickEvent;
Var
 DT : TKADaoTable;
Begin
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active) Then
     Begin
      DT := TKaDaoTable(DataSource.Dataset);
      Application.CreateForm(TKADaoSortByDialog,KADaoSortByDialog);
      if KADaoSortByDialog.Execute(DT.SortFieldNames,DT.SortedBy,DT.UseBrackets) Then
       Begin
         DT.Sort;
       End;
      KADaoSortByDialog.Free;
     End;
End;

Procedure TKADaoSortByButton.Click;
Begin
 if FBefore Then inherited Click;
 HandleClickEvent;
 if NOT FBefore Then inherited Click;
End;

procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKADaoSortByButton]);
end;

end.
